@@ -21,6 +21,16 @@ function sq_new_function()
 
 
 
+// constants added in version 1.5.2
+//
+define('SM_DEBUG_MODE_OFF', 0);             // complete error suppression
+define('SM_DEBUG_MODE_SIMPLE', 1);          // PHP E_ERROR
+define('SM_DEBUG_MODE_MODERATE', 512);      // PHP E_ALL
+define('SM_DEBUG_MODE_ADVANCED', 524288);   // PHP E_ALL plus log errors intentionally suppressed
+define('SM_DEBUG_MODE_STRICT', 536870912);  // PHP E_STRICT
+
+
+
 //
 // taken from functions/global.php on 2007/06/23
 // since 1.5.2
@@ -271,14 +281,15 @@ function check_plugin_version($plugin_name,
 
 
 //
-// taken from functions/plugin.php on 2007/06/29
+// taken from functions/plugin.php on 2008/07/22
 // since 1.5.2
 //
 if (!function_exists('get_plugin_requirement'))
 {
 function get_plugin_requirement($plugin_name, $requirement,
+                                $ignore_incompatible = TRUE,
                                 $force_inclusion = FALSE)
-{                               
+{
 
    $info_function = $plugin_name . '_info';
    $plugin_info = array();
@@ -374,10 +385,19 @@ function get_plugin_requirement($plugin_name, $requirement,
          $b = $version_array[1];
          $c = $version_array[2];
 
+         // complicated way to say we are interested in these overrides
+         // if the version is applicable to us and if the overrides include
+         // the requirement we are looking for, or if the plugin is not
+         // compatible with this version of SquirrelMail (unless we are
+         // told to ignore such)
+         //
          if (check_sm_version($a, $b, $c)
-          && ( !empty($requirement_overrides[SQ_INCOMPATIBLE])
-          || (isset($requirement_overrides[$requirement])
-          && !is_null($requirement_overrides[$requirement]))))
+          && ((!$ignore_incompatible
+            && (!empty($requirement_overrides[SQ_INCOMPATIBLE])
+             || $requirement_overrides === SQ_INCOMPATIBLE))
+           || (is_array($requirement_overrides)
+            && isset($requirement_overrides[$requirement])
+            && !is_null($requirement_overrides[$requirement]))))
          {
 
             if (empty($highest_version_array)
@@ -389,7 +409,8 @@ function get_plugin_requirement($plugin_name, $requirement,
              && $highest_version_array[2] < $c))
             {
                $highest_version_array = $version_array;
-               if (!empty($requirement_overrides[SQ_INCOMPATIBLE]))
+               if (!empty($requirement_overrides[SQ_INCOMPATIBLE])
+                || $requirement_overrides === SQ_INCOMPATIBLE)
                   $requirement_value_override = SQ_INCOMPATIBLE;
                else
                   $requirement_value_override = $requirement_overrides[$requirement];
@@ -790,3 +811,114 @@ function get_secured_config_value($var_name) {
 
 
 
+//
+// taken from functions/compose.php on 2008/02/29
+// since 1.5.2
+//
+if (!function_exists('sq_send_mail'))
+{
+function sq_send_mail($to, $subject, $body, $from, $cc='', $bcc='', $message='')
+{
+
+   require_once(SM_PATH . 'functions/mime.php');
+   require_once(SM_PATH . 'class/mime.class.php');
+
+   if (empty($message))
+   {
+      $message = new Message();
+      $header  = new Rfc822Header();
+
+      $message->setBody($body);
+      $content_type = new ContentType('text/plain');
+      global $special_encoding, $default_charset;
+      if ($special_encoding)
+         $rfc822_header->encoding = $special_encoding;
+      else
+         $rfc822_header->encoding = '8bit';
+      if ($default_charset)
+         $content_type->properties['charset']=$default_charset;
+      $header->content_type = $content_type;
+
+      $header->parseField('To', $to);
+      $header->parseField('Cc', $cc);
+      $header->parseField('Bcc', $bcc);
+      $header->parseField('From', $from);
+      $header->parseField('Subject', $subject);
+      $message->rfc822_header = $header;
+   }
+//sm_print_r($message);exit;
+
+
+   global $useSendmail;
+
+
+   // ripped from src/compose.php - based on both 1.5.2 and 1.4.14
+   //
+   if (!$useSendmail) {
+      require_once(SM_PATH . 'class/deliver/Deliver_SMTP.class.php');
+      $deliver = new Deliver_SMTP();
+      global $smtpServerAddress, $smtpPort, $pop_before_smtp,
+             $domain, $pop_before_smtp_host;
+
+      $authPop = (isset($pop_before_smtp) && $pop_before_smtp) ? true : false;
+      if (empty($pop_before_smtp_host)) $pop_before_smtp_host = $smtpServerAddress;
+      $user = '';
+      $pass = '';
+      get_smtp_user($user, $pass);
+      $stream = $deliver->initStream($message,$domain,0,
+                $smtpServerAddress, $smtpPort, $user, $pass, $authPop, $pop_before_smtp_host);
+   } else {
+      require_once(SM_PATH . 'class/deliver/Deliver_SendMail.class.php');
+      global $sendmail_path, $sendmail_args;
+      // Check for outdated configuration
+      if (!isset($sendmail_args)) {
+         if ($sendmail_path=='/var/qmail/bin/qmail-inject') {
+            $sendmail_args = '';
+         } else {
+            $sendmail_args = '-i -t';
+         }
+      }
+      $deliver = new Deliver_SendMail(array('sendmail_args'=>$sendmail_args));
+      $stream = $deliver->initStream($message,$sendmail_path);
+   }
+
+
+   $success = false;
+   $message_id = '';
+   if ($stream) {
+      $deliver->mail($message, $stream);
+      if (!empty($message->rfc822_header->message_id)) {
+         $message_id = $message->rfc822_header->message_id;
+      }
+
+      $success = $deliver->finalizeStream($stream);
+   }
+
+   return array($success, $message_id);
+
+}
+}
+
+
+
+//
+// taken from functions/html.php on 2008/05/21
+// since 1.5.2
+//
+if (!function_exists('set_uri_vars'))
+{
+function set_uri_vars($uri, $values, $sanitize=TRUE) {
+    foreach ($values as $key => $value)
+        if (is_array($value)) {
+          $i = 0;
+          foreach ($value as $val)
+             $uri = set_url_var($uri, $key . '[' . $i++ . ']', $val, $sanitize);
+        }
+        else
+          $uri = set_url_var($uri, $key, $value, $sanitize);
+    return $uri;
+}
+}
+
+
+
